C******** ResQED ********
C
C     This program performs symbolic computations with operators in
C     Quantum Electrodynamics
C
C     Code compilation:
C       just type 'make' in this directory to compile the code.
C
C     Usage:
C       cmt.exe < inp > out &
C
C     inp ... input file which contains a series of instructions telling
C         1. where (file name) to take operator(s);
C         2. what to do with them (operation);
C         3. where (file name) to put the result of operation.
C     Example:
C
C file: inp
C*********************
C      0                                                    | Parameter telling how to print
C Grand calculation FD renormalized S-matrix in 4th order   | Comments
CINT                                                        | Keyword INT tells that 'integration' is performed
CV                   Vint                                   | operator from file V is 'integrated and result is written in file Vint
CPRO                                                        | Keyword PRO tells that 'normal product' of two operators will be found
CV                   Vint               VxVint              | Calculate 'normal product' of operators in V and Vint, result in VxVint
C*********************

COperators are contained in 'operator files' in the following format.

C     Example:
C File VxU2xVintint:                                line #
C*********************
C 1 1 3 4 -6 1 4 0                               |  1
C U(+h*)U(+g*)d*d*f*f*                           |  2
C 5 6 7 8 8 5 6 3 2 4 1 7                        |  3
C +[+Om(+g*-f*-d*)-Om(+f*)-Os(+g*-d*)]           |  4
C +[+Om(+g*-f*-d*)-Om(+f*)-Os(+g*-d*)]           |  5
C +[+Om(+h*)-Om(+g*)+Os(+h*-g*)]                 |  6 
C +[+Os(+h*-g*)]                                 |  7
C +[+Os(+g*-d*)]                                 |  8
C x                                              |  9
C +Ei-[+Om(+g*-f*-d*)+Om(+g*)-Om(+f*)-Om(+d*)]t  | 10
C x                                              | 11
C A^(+d*)                                        | 12
C 1 0                                            | 13
C A^(+f*)                                        | 14
C 2 0                                            | 15
C A_(+g*)                                        | 16
C 3 0                                            | 17
C A_(-g*+f*+d*)                                  | 18
C 4 0                                            | 19
C x                                              | 20
C 1 1 3 4 -6 1 4 0
C U(+h*)U(+d*)d*d*f*f*
C 5 6 7 8 8 3 2 4 6 7 1 5
C +[+Om(+h*)-Om(+d*)+Os(+h*-d*)]
C +[+Om(+g*)-Om(+d*)-Os(+g*-d*)]
C +[+Om(+g*)-Om(+d*)-Os(+g*-d*)]
C +[+Os(+h*-d*)]
C +[+Os(+g*-d*)]
C x
C +Ei-[+Om(+g*-f*-d*)+Om(+g*)-Om(+f*)-Om(+d*)]t
C x
C A^(+d*)
C 1 0
C A^(+f*)
C 2 0
C A_(+g*)
C 3 0
C A_(-g*+f*+d*)
C 4 0
C x
C*****************************

C  This file describes an operator which is a sum of two terms. The first
C  term is described in first 20 lines. Then goes the second term. Let's see
C  how the first term is described line by line.
C
C  Each term is a product of a 
C  - numerical factor
C  - power of i
C  - power of electron charge (e)
C  - power of pi
C  - electron creation/destruction operators;
C  - positron creation/destruction operators;
C  - photon creation/destruction operators;
C  - electron propagation function U;
C  - positron propagation function V;
C  - gamma matrices
C  - energy denominators;
C  - time exponent;
C
C line 1: 
C 1 1 3 4 -6 1 4 0
C        IPowr(1) ... factor numerator
C        IPowr(2) ... factor denumerator
C        IPowr(3) ... power of i
C        IPowr(4) ... power of e
C        IPowr(5) ... power of pi numerator
C        IPowr(6) ... power of pi denumerator
C        IPowr(7) ... divergence index: double of power of the
C                     integration variable in denominator
C        IPowr(8) ... not used
C

C line 2 and 3:
C U(+h*)U(+g*)d*d*f*f*                           
C 5 6 7 8 8 5 6 3 2 4 1 7 
C
C This can be rewritten in a more familar notation (LaTeX)
C
C U(h)_{56}U(g)_{78}\gamma^{\mu}_{85}\gamma_{\mu63}\gamma^{\nu}_{24}\gamma_{\nu17}
C line 3 just lists indices for factors in line 2

C line 4 - 8:
C energy denominators, e.g., line 4 describes factor
C
C + \frac{1}{\omega_{g-f-d} - \omega_{f} - |g-d|},
C
C where \omega_{p} = \sqrt{p^2 + m^2} denoted 'Om'
C                    |p|              denoted 'Os'
C                    \sqrt{|p|}       denoted 'Oz'

C line 9:
C x means that there are no more energy denominators

C line 10:
C +Ei-[+Om(+g*-f*-d*)+Om(+g*)-Om(+f*)-Om(+d*)]t
C
C time exponent
C
C e^{-iEt}, where E is the energy in the brackets
C           written in the same format as energy denominators

C line 11:
C x means that there is no more time exponents (probably not needed)

C lines 12-19:
C particle creation/destruction operators, e.g.
C
C A^(+d*)                                       
C 1 0                                            
C
C is electron creation operator a^{\dag}_1 (d)

 

      Program main
      Use ModXpr
      Use ModTem
      Implicit None
      Integer NOpern
C     NOpern  ... number of instructions in the input file
      Parameter(NOpern=1500)

      Character*3 KeyWrd,KWord2
      Character*2 Order
      Character*20 FileN1,FileN2,FilOut

      Character*80 Title
      Integer IType,IPrint,ITempl
      Integer INum,IDen,IPow
      Integer j,i
      COMMON /Type/ IType
      COMMON /Print/ IPrint

C     Possible keywords are:
C     COM ... take commutator and normal ordering
C     SUM ... take sum 
C     SOL ... solve commutator with H0
C     MUL ... multiply by a constant
C     MAT ... matrix element
C     PRO ... product of two operators and normal ordering
C     ENR ... take operators on the energy shell
C     DEL ... delete terms given in templates
 
      Read(5,*)IPrint
      Write(6,*) 'IPrint =',IPrint
      NTempl=0
      Read(5,*) Title
      Write(6,*) Title


      Do i=1,NOpern
         Read(5,FMT=22,END=100)KeyWrd,Order
         If(KeyWrd.eq.'   ') Goto 100
         Write(6,*)KeyWrd,Order
 22      Format(A3,A2)
C        Decide what is the order of the perturbation theory
         OrderN = 100
         If(Order.eq.' 2') then
            OrderN = 2
         Else If(Order.eq.' 3') then
            OrderN = 3
         Else If(Order.eq.' 4') then
            OrderN = 4
         End If
         NTempl=0
         If(KeyWrd.eq.'COM') then
            Read(5,*) FileN1,FileN2,FilOut
            Write(6,*) FileN1,FileN2,FilOut
            IType=0
            Read(5,*)KWord2
            Write(6,*) KWord2
            If(KWord2.eq.'FIL') Call ReadTm()
         Else If(KeyWrd.eq.'SUM') then
            Read(5,*) FileN1,FileN2,FilOut
            Write(6,*) FileN1,FileN2,FilOut
            IType=1
         Else If(KeyWrd.eq.'SOL') then
            Read(5,*) FileN1,FilOut
            Write(6,*) FileN1,FilOut
            IType=2
         Else If(KeyWrd.eq.'MUL') then
            Read(5,*) FileN1,FilOut
            Write(6,*)'Input file  ',FileN1
            Write(6,*)'Output file  ',FilOut
            Read(5,*) INum,IDen,IPow
            IType=3
            Read(5,*)KWord2
            Write(6,*) KWord2
            If(KWord2.eq.'FIL') Call ReadTm()
         Else If(KeyWrd.eq.'MAT') then
            Read(5,*) FileN1,FileN2,FilOut
            Write(6,*)'Input files  ',FileN1,FileN2
            Write(6,*)'Output file  ',FilOut
            IType=4
            NTempl=1
            Templt(1)%Length = 0
         Else If(KeyWrd.eq.'PRO') then
            Read(5,*) FileN1,FileN2,FilOut
            Write(6,*) FileN1,FileN2,FilOut
            IType=5
            Read(5,*)KWord2
            Write(6,*) KWord2
            If(KWord2.eq.'FIL') Call ReadTm()
         Else If(KeyWrd.eq.'ENR') then
            Read(5,*) FileN1,FilOut
            Write(6,*)'Input file  ',FileN1
            Write(6,*)'Output file  ',FilOut
            IType=6
         Else If(KeyWrd.eq.'DEL') then
            Read(5,*) FileN1,FilOut
            Write(6,*)'Input file  ',FileN1
            Write(6,*)'Output file  ',FilOut
            IType=7
            Call ReadTm()
         Else If(KeyWrd.eq.'IN2') then
C           Calculate integral of an expression
            Read(5,*) FileN1,FilOut
            Write(6,*)'Input file  ',FileN1
            Write(6,*)'Output file  ',FilOut
            IType=8
         Else If(KeyWrd.eq.'CON') then
C           Remove unconnected diagrams (after PRO)
            Read(5,*) FileN1,FilOut
            Write(6,*)'Input file  ',FileN1
            Write(6,*)'Output file  ',FilOut
            IType=9
         Else If(KeyWrd.eq.'INF') then
C           Take time t to + infinity )T)
            Read(5,*) FileN1,FilOut
            Write(6,*)'Input file  ',FileN1
            Write(6,*)'Output file  ',FilOut
            IType=10
        Else If(KeyWrd.eq.'EN2') then
C           Alternative way to go to the mass shell
            Read(5,*) FileN1,FilOut
            Write(6,*)'Input file  ',FileN1
            Write(6,*)'Output file  ',FilOut
            IType=11
         Else If(KeyWrd.eq.'INT') then
C           Calculate integral of an expression
C           Same as INT but no T-dependent term is created
            Read(5,*) FileN1,FilOut
            Write(6,*)'Input file  ',FileN1
            Write(6,*)'Output file  ',FilOut
            IType=12
         Else If(KeyWrd.eq.'DLT') then
C           Remove delta functions
            Read(5,*) FileN1,FilOut
            Write(6,*)'Input file  ',FileN1
            Write(6,*)'Output file  ',FilOut
            IType=13
        Else
            Write(6,*) 'Warning!!! Unknown keyword!',KeyWrd
        End If

C         Write(6,*) 'Call Master'
C         Write(6,*) 'NTempl = ',NTempl
         Call Master(INum,IDen,IPow,FileN1,FileN2,FilOut)
      End Do
 100  Continue
      Stop 
      End
