      Subroutine Normal(ICurr,Exp3,ExpR)

C     This program takes a term (Exp3)
C     transforms it to the normal form and writes all obtained
C     terms into the array of results (ExpR) 
C     ICurr ... current number of terms already included in ExpR array

      Use ModXpr
      Use Md1Opr
      Implicit None
      Type(Expression)                    :: Exp3
      Type(Expression), Dimension(NumExp) :: ExpR
      
C     Internal arrays
      Type(One_Operator) Op
      Integer  NElCre,NElDes,NPoCre,NPoDes,NPhCre,NPhDes  
      Integer ICurr,ITrms,ITerms,IPostn,IRefer,ICrOld,i,j
      Logical Electron_Creation,Electron_Destruction
      Logical Positron_Creation,Positron_Destruction
      Logical Photon_Creation,Photon_Destruction
      Logical Electron_Operator, Positron_Operator
      Logical Fermion_Operator
      Integer IType
      COMMON /Type/ IType
      Integer IPrint
      COMMON /Print/ IPrint

      If(IPrint.ge.9) Call Output(6,Exp3,'Before GetRid',-1)

C     Get rid of '$' momenta in the computed commutator
      Call GetRid(Exp3)
      If(IPrint.ge.10) Call Output(6,Exp3,'After GetRid',-1)


C     Next: put Oper3 into normal form
C     Original Oper3 will be put at position ICurr in 'R' 
C     arrays
C     Additional terms produced by pairing of operators during 
C     conversion of Oper3 to the normal form will start from 
C     ICrOld
      ICrOld=ICurr+1
C     ICrOld is the position at which the term after
C     pairing will stay. Terms from ICrOld to ICurr
C     are related to the ICrOld, ie. produced by converting
C     ICrOld into the normal form

      If(IType.eq.4) then
         NElCre=0
         NElDes=0
         NPoCre=0
         NPoDes=0
         NPhCre=0
         NPhDes=0         
         Do i=1,Exp3%Oper%NumOp
            Op=Exp3%Oper%OperG(i)
            If(Electron_Creation(Op))    NElCre=NElCre+1
            If(Electron_Destruction(Op)) NElDes=NElDes+1
            If(Positron_Creation(Op))    NPoCre=NElCre+1
            If(Positron_Destruction(Op)) NPoDes=NElDes+1
            If(Photon_Creation(Op))      NPhCre=NElCre+1
            If(Photon_Destruction(Op))   NPhDes=NElDes+1
         End Do
         If(NElCre.ne.NElDes) Return
         If(NPoCre.ne.NPoDes) Return
         If(NPhCre.ne.NPhDes) Return
      End If

C     Increment ICurr
      ICurr=ICurr+1
C     Fill new position in 'R' arrays with '3' arrays
         ExpR(ICurr) = Exp3
C     Now at ICurr is a term which should be converted to a normal form
      If(IPrint.ge.5) 
     $Call Output(6,Exp3,'Trying to convert to normal form the term',-1)
      ITrms=0
      ITerms=ICurr
      If(ITerms.ge.NumExp) Write(6,*)
     $'Warning!!! Number of terms exceeded allowed limit  ', NumExp
C     Loop over all terms produced by the current Oper3
 110  Continue
C     ICurr   ... the first element which is not yet in the normal form
C     ITerms ... last term written to the 'R' arrays
C     The do loop should go from ICurr to ITerms
C      We use ITrms to avoid changing the loop limit inside loop
      ITrms=ITerms
      Do i=ICurr,ITrms
C        First move all A^ to the leftmost positions
 20      Continue
         Do j=2,ExpR(i)%Oper%NumOp
            If(.not.Electron_Creation(ExpR(i)%Oper%OperG(j  ))) Cycle
            If(     Electron_Creation(ExpR(i)%Oper%OperG(j-1))) Cycle
      
            Call ExchangeOperators(ExpR(i),ExpR,ITerms,j)
            Goto 20            
         End Do  
C        Next move all A_ to the leftmost (but to the right from A^ 
C        positions
 40      Continue
         Do j=2,ExpR(i)%Oper%NumOp
            If(.not.Electron_Destruction(ExpR(i)%Oper%OperG(j))) Cycle
            If(Electron_Operator(ExpR(i)%Oper%OperG(j-1))) Cycle

            Call ExchangeOperators(ExpR(i),ExpR,ITerms,j)
            Goto 40
         End Do
C        Next move all B^ to the leftmost (but to the right from all A) 
C        positions
  60     Continue
         Do j=2,ExpR(i)%Oper%NumOp
            If(.not.Positron_Creation(ExpR(i)%Oper%OperG(j))) Cycle
            If( Electron_Operator(ExpR(i)%Oper%OperG(j-1))
     $      .or.Positron_Creation(ExpR(i)%Oper%OperG(j-1))) Cycle
            
            Call ExchangeOperators(ExpR(i),ExpR,ITerms,j)
            Goto 60
         End Do
C        Next move all B_ to the leftmost (but to the right from all A 
C        and B^) positions
  80     Continue
         Do j=2,ExpR(i)%Oper%NumOp
            If(.not.Positron_Destruction(ExpR(i)%Oper%OperG(j))) Cycle
            If(Fermion_Operator(ExpR(i)%Oper%OperG(j-1))) Cycle

            Call ExchangeOperators(ExpR(i),ExpR,ITerms,j)
            Goto 80
         End Do
C        Next move all C^ to the leftmost (but to the right from all 
C        A and B) positions
 100     Continue
         Do j=2,ExpR(i)%Oper%NumOp
            If(.not.Photon_Creation(ExpR(i)%Oper%OperG(j)))  Cycle
            If(Photon_Creation( ExpR(i)%Oper%OperG(j-1))
     $      .or.Fermion_Operator( ExpR(i)%Oper%OperG(j-1))) Cycle

            Call ExchangeOperators(ExpR(i),ExpR,ITerms,j)
            Goto 100
         End Do
C        Now the terms from ICurr to ITrms are OK,however, 
C        newly created terms from
C        ITrms+1 to ITerms are not
C        OK.  We should repeat the same procedure for them.
      End Do
      ICurr=ITrms+1
C      Write(6,*) 'After do loop ICurr=',ICurr
      If(ITerms.ge.ICurr) goto 110
      ICurr=ICurr-1
C     If(ITerms.lt.ICurr) means we haven't produced new pairings,
C     so we can quit.
      Return
      End

